@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HTML-TEMPLATE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/html-template/template.lisp,v 1.14 2003/07/15 12:51:00 edi Exp $
+;;; $Header: /usr/local/cvsrep/html-template/template.lisp,v 1.15 2003/07/15 19:07:52 edi Exp $
 
 ;;; Copyright (c) 2003, Dr. Edmund Weitz. All rights reserved.
 
@@ -29,6 +29,24 @@
 
 (in-package #:html-template)
 
+(defmacro with-use-value-restart ((symbol) error-form)
+  "Provide a USE-VALUE restart for ERROR-FORM in case the value
+associated with SYMBOL isn't to our liking."
+  `(restart-case
+      ,error-form
+    (use-value (other-value)
+      :report (lambda (stream)
+                (format stream
+                        "Use another value for symbol ~S: "
+                        ,symbol))
+      :interactive (lambda ()
+                     (format t
+                             "Enter another value for symbol ~S: "
+                             ,symbol)
+                     (multiple-value-list (eval (read))))
+      other-value)))
+    
+
 (defun create-simple-printer (string-list &optional symbol/pathname (next-fn #'no-values))
   "Used internally to create template printers for TMPL_VAR,
 TMPL_INCLUDE, and for strings which don't include template
@@ -47,40 +65,24 @@ printed first."
         ;; TMPL_VAR tag
         (lambda (values)
           (write-string string)
-          (write-string (restart-case
-                            (let ((value (funcall *value-access-function*
-                                                  symbol/pathname values)))
-                              (typecase value
-                                (null
-                                  (if *convert-nil-to-empty-string*
-                                    ""
-                                    (signal-template-missing-value-error
-                                     "Value for symbol ~S is NIL"
-                                     symbol/pathname)))
-                                (string
-                                  value)
-                                (otherwise
-                                  (error 'template-not-a-string-error
-                                         :value value
-                                         :format-control "Value ~S for symbol ~S is not a string"
-                                         :format-arguments (list value symbol/pathname)))))
-                          ;; provide a restart in case the associated
-                          ;; value isn't to our liking
-                          (other-value (other-value)
-                            :report (lambda (stream)
-                                      (format stream
-                                              "Use another value for symbol ~S: "
-                                              symbol/pathname))
-                            :interactive (lambda ()
-                                           (format t
-                                                   "Enter another value for symbol ~S: "
-                                                   symbol/pathname)
-                                           (multiple-value-list (eval (read))))
-                            :test (lambda (condition)
-                                    (typep condition '(or null
-                                                       template-not-a-string-error
-                                                       template-missing-value-error)))
-                            other-value)))
+          (write-string (let ((value (funcall *value-access-function*
+                                              symbol/pathname values)))
+                          (typecase value
+                            (null
+                              (if *convert-nil-to-empty-string*
+                                ""
+                                (with-use-value-restart (symbol/pathname)
+                                  (signal-template-missing-value-error
+                                   "Value for symbol ~S is NIL"
+                                   symbol/pathname))))
+                            (string
+                              value)
+                            (otherwise
+                              (with-use-value-restart (symbol/pathname)
+                                (error 'template-not-a-string-error
+                                       :value value
+                                       :format-control "Value ~S for symbol ~S is not a string"
+                                       :format-arguments (list value symbol/pathname)))))))
           (funcall next-fn values)))
       (pathname
         ;; TMPL_INCLUDE tag
